# frozen_string_literal: true

# Copyright (c) [2023] SUSE LLC
#
# All Rights Reserved.
#
# This program is free software; you can redistribute it and/or modify it
# under the terms of version 2 of the GNU General Public License as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
# more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, contact SUSE LLC.
#
# To contact SUSE LLC about this file by physical or electronic mail, you may
# find current contact information at www.suse.com.

require "fileutils"
require "yast"
require "ostruct"
require "suse/connect"
require "y2packager/new_repository_setup"
require "y2packager/resolvable"

require "agama/cmdline_args"
require "agama/registered_addon"

Yast.import "Arch"
Yast.import "Pkg"

module Agama
  # Handles everything related to registration of system to SCC, RMT or similar.
  class Registration
    # NOTE: identical and keep in sync with Software::Manager::TARGET_DIR
    TARGET_DIR = "/run/agama/zypp"
    private_constant :TARGET_DIR

    # FIXME: it should use TARGET_DIR instead of "/", but connect failed to read it even
    # if fs_root passed as client params. Check with SCC guys why.
    GLOBAL_CREDENTIALS_PATH = File.join("/",
      SUSE::Connect::YaST::GLOBAL_CREDENTIALS_FILE)
    private_constant :GLOBAL_CREDENTIALS_PATH

    # Code used for registering the product.
    #
    # @return [String, nil] nil if the product is not registered yet.
    attr_reader :reg_code

    # Email used for registering the product.
    #
    # @return [String, nil]
    attr_reader :email

    # List of already registered addons
    #
    # @return [Array<RegisteredAddon>]
    attr_reader :registered_addons

    # @param software_manager [Agama::Software::Manager]
    # @param logger [Logger]
    def initialize(software_manager, logger)
      @software = software_manager
      @logger = logger
      @services = []
      @credentials_files = []
      @registered_addons = []
    end

    # Registers the selected product.
    #
    # @raise [
    #   SocketError|Timeout::Error|SUSE::Connect::ApiError|
    #   SUSE::Connect::MissingSccCredentialsFile|SUSE::Connect::MissingSccCredentialsFile|
    #   OpenSSL::SSL::SSLError|JSON::ParserError
    # ]
    #
    # @param code [String] Registration code.
    # @param email [String] Email for registering the product.
    def register(code, email: "")
      return if product.nil? || reg_code

      reg_params = connect_params(token: code, email: email)

      @login, @password = SUSE::Connect::YaST.announce_system(reg_params, target_distro)
      # write the global credentials
      # TODO: check if we can do it in memory for libzypp
      SUSE::Connect::YaST.create_credentials_file(@login, @password, GLOBAL_CREDENTIALS_PATH)

      target_product = OpenStruct.new(
        arch:       Yast::Arch.rpm_arch,
        identifier: product.id,
        version:    product.version || "1.0"
      )
      activate_params = {}
      service = SUSE::Connect::YaST.activate_product(target_product, activate_params, email)
      process_service(service)

      @reg_code = code
      @email = email
      run_on_change_callbacks
    end

    def register_addon(name, version, code)
      if @registered_addons.any? { |a| a.name == name && a.version == version }
        @logger.info "Addon #{name}-#{version} already registered, skipping registration"
        return
      end

      @logger.info "Registering addon #{name}-#{version}"
      # do not log the code, but at least log if it is empty
      @logger.info "Using empty registration code" if code.empty?

      target_product = OpenStruct.new(
        arch:       Yast::Arch.rpm_arch,
        identifier: name,
        version:    version
      )
      activate_params = { token: code }
      service = SUSE::Connect::YaST.activate_product(target_product, activate_params, @email)
      process_service(service)

      @registered_addons << RegisteredAddon.new(name, version, code)
      # select the products to install
      @software.addon_products(find_addon_products)

      run_on_change_callbacks
    end

    # Deregisters the selected product.
    #
    # It uses the registration code and email passed to {#register}.
    #
    # @raise [
    #   SocketError|Timeout::Error|SUSE::Connect::ApiError|
    #   SUSE::Connect::MissingSccCredentialsFile|SUSE::Connect::MissingSccCredentialsFile|
    #   OpenSSL::SSL::SSLError|JSON::ParserError
    # ]
    def deregister
      return unless reg_code

      @services.each do |service|
        Y2Packager::NewRepositorySetup.instance.services.delete(service.name)
        @software.remove_service(service)
      end

      # reset
      @software.addon_products([])
      @services = []

      reg_params = connect_params(token: reg_code, email: email)
      SUSE::Connect::YaST.deactivate_system(reg_params)
      FileUtils.rm(GLOBAL_CREDENTIALS_PATH) # connect does not remove it itself
      @credentials_files.each do |credentials_file|
        FileUtils.rm(File.join(TARGET_DIR, credentials_path(credentials_file)))
      end
      @credentials_files = []

      @reg_code = nil
      @email = nil
      @registered_addons = []

      run_on_change_callbacks
    end

    # Copies credentials files to the target system.
    def finish
      return unless reg_code

      files = [[
        GLOBAL_CREDENTIALS_PATH, File.join(Yast::Installation.destdir, GLOBAL_CREDENTIALS_PATH)
      ]]
      @credentials_files.each do |credentials_file|
        files << [
          File.join(TARGET_DIR, credentials_path(credentials_file)),
          File.join(Yast::Installation.destdir, credentials_path(credentials_file))
        ]
      end

      files.each do |src_dest|
        FileUtils.cp(*src_dest)
      end
    end

    # Callbacks to be called when registration changes (e.g., a different product is selected).
    def on_change(&block)
      @on_change_callbacks ||= []
      @on_change_callbacks << block
    end

  private

    # @return [Agama::Software::Manager]
    attr_reader :software

    # Currently selected product.
    #
    # @return [Agama::Software::Product, nil]
    def product
      software.product
    end

    # Product name expected by SCC.
    #
    # @return [String] E.g., "ALP-Dolomite-1-x86_64".
    def target_distro
      v = product.version.to_s.split(".").first || "1"
      "#{product.id}-#{v}-#{Yast::Arch.rpm_arch}"
    end

    def run_on_change_callbacks
      @on_change_callbacks&.map(&:call)
    end

    # taken from https://github.com/yast/yast-registration/blob/master/src/lib/registration/url_helpers.rb#L109
    def credentials_from_url(url)
      parsed_url = URI(url)
      params = URI.decode_www_form(parsed_url.query).to_h

      params["credentials"]
    rescue StandardError
      # if something goes wrong try to continue like if there is no credentials param
      nil
    end

    def credentials_path(file)
      File.join(SUSE::Connect::YaST::DEFAULT_CREDENTIALS_DIR, file)
    end

    # Returns the arguments to connect to the registration server
    #
    # @param params [Hash] additional parameters (e.g., email and token)
    # @return [Hash]
    def connect_params(params = {})
      default_params = {}
      default_params[:url] = registration_url if registration_url
      default_params.merge(params)
    end

    # Returns the URL of the registration server
    #
    # At this point, it just checks the kernel's command-line.
    #
    # @return [String, nil]
    def registration_url
      cmdline_args = CmdlineArgs.read
      cmdline_args.data["register_url"]
    end

    # process a newly added service, create the credentials file and add the service to libzypp
    def process_service(service)
      @services << service
      credentials_file = credentials_from_url(service.url)
      if credentials_file
        @credentials_files << credentials_file
        # addons use the same SCC credentials as the base product
        SUSE::Connect::YaST.create_credentials_file(@login, @password,
          File.join(TARGET_DIR, credentials_path(credentials_file)))
      end
      Y2Packager::NewRepositorySetup.instance.add_service(service.name)
      @software.add_service(service)
    end

    # Find all addon products
    #
    # @return [Array<String>] names of the products
    def find_addon_products
      # find all repositories for registered addons (their services)
      addon_repos = @services.reduce([]) do |acc, service|
        # skip the first service, it belongs to the base product
        next acc if service == @services.first

        acc.concat(service_repos(service))
      end

      # find all products from those repositories
      products = Y2Packager::Resolvable.find(kind: :product)
      products.select! do |product|
        addon_repos.any? { |addon_repo| product.source == addon_repo["SrcId"] }
      end

      products.map!(&:name)
      @logger.info "Addon products to install: #{products}"

      products
    end

    # Find all repositories belonging to a service.
    #
    # @param product_service [OpenStruct] repository service from suseconnect
    #
    # @return [Array<Hash>] repository data as returned by the Pkg.SourceGeneralData
    #   call, additionally with the "SrcId" key
    def service_repos(product_service)
      @logger.info "product_service: #{product_service.inspect}"
      repo_data = Yast::Pkg.SourceGetCurrent(false).map { |repo| repository_data(repo) }

      service_name = product_service.name
      # select only repositories belonging to the product services
      repos = repo_data.select { |repo| service_name == repo["service"] }
      @logger.info "Service #{service_name.inspect} repositories: #{repos}"

      repos
    end

    # Get repository data
    # @param [Fixnum] repo repository ID
    # @return [Hash] repository properties, including the repository ID ("SrcId" key)
    def repository_data(repo)
      data = Yast::Pkg.SourceGeneralData(repo)
      data["SrcId"] = repo
      data
    end
  end
end
